@@ -20,8 +20,6 @@ x: [bar(int128), baz(baffle)]
     """
 struct A:
     b: B
-struct B:
-    a: A
     """,
 ]
 
@@ -38,12 +36,25 @@ invalid_list = [
 def foo():
     raw_log(b"cow", b"dog")
     """,
+    """
+@external
+def foo():
+    xs: uint256[1] = []
+    """,
     # Must be a literal string.
     """
 @external
 def mint(_to: address, _value: uint256):
     assert msg.sender == self,msg.sender
     """,
+    # literal longer than event member
+    """
+event Foo:
+    message: String[1]
+@external
+def foo():
+    log Foo("abcd")
+    """,
     # Raise reason must be string
     """
 @external
