@@ -1,7 +1,3 @@
-import pytest
-from eth_tester.exceptions import TransactionFailed
-
-
 def test_assure_refund(w3, get_contract):
     code = """
 @external
@@ -13,7 +9,7 @@ def foo():
     a0 = w3.eth.accounts[0]
     gas_sent = 10 ** 6
     tx_hash = c.foo(transact={"from": a0, "gas": gas_sent, "gasPrice": 10})
-    tx_receipt = w3.eth.getTransactionReceipt(tx_hash)
+    tx_receipt = w3.eth.get_transaction_receipt(tx_hash)
 
     assert tx_receipt["status"] == 0
     assert tx_receipt["gasUsed"] == gas_sent  # Drains all gains sent
@@ -32,13 +28,9 @@ def foo(val: int128) -> bool:
 
     assert c.foo(2) is True
 
-    assert_tx_failed(lambda: c.foo(1))
-    assert_tx_failed(lambda: c.foo(-1))
-
-    with pytest.raises(TransactionFailed) as e_info:
-        c.foo(-2)
-
-    assert "Invalid opcode 0xfe" in e_info.value.args[0]
+    assert_tx_failed(lambda: c.foo(1), exc_text="Invalid opcode 0xfe")
+    assert_tx_failed(lambda: c.foo(-1), exc_text="Invalid opcode 0xfe")
+    assert_tx_failed(lambda: c.foo(-2), exc_text="Invalid opcode 0xfe")
 
 
 def test_basic_call_unreachable(w3, get_contract, assert_tx_failed):
@@ -59,6 +51,6 @@ def foo(val: int128) -> int128:
 
     assert c.foo(33) == -123
 
-    assert_tx_failed(lambda: c.foo(1))
-    assert_tx_failed(lambda: c.foo(1))
-    assert_tx_failed(lambda: c.foo(-1))
+    assert_tx_failed(lambda: c.foo(1), exc_text="Invalid opcode 0xfe")
+    assert_tx_failed(lambda: c.foo(1), exc_text="Invalid opcode 0xfe")
+    assert_tx_failed(lambda: c.foo(-1), exc_text="Invalid opcode 0xfe")
