@@ -1,13 +1,17 @@
 import math
 from dataclasses import dataclass
-from typing import Optional
+from typing import Dict, Optional
 
 from vyper import ast as vy_ast
+from vyper.address_space import MEMORY
+from vyper.codegen.ir_node import Encoding
+from vyper.codegen.types import NodeType, parse_type
 from vyper.exceptions import StructureException
-from vyper.old_codegen.lll_node import Encoding
-from vyper.old_codegen.types import NodeType, canonicalize_type, get_size_of_type, parse_type
 from vyper.utils import cached_property, mkalphanum
 
+# dict from function names to signatures
+FunctionSignatures = Dict[str, "FunctionSignature"]
+
 
 # Function variable
 # TODO move to context.py
@@ -20,7 +24,7 @@ class VariableRecord:
         typ,
         mutable,
         encoding=Encoding.VYPER,
-        location="memory",
+        location=MEMORY,
         blockscopes=None,
         defined_at=None,
         is_internal=False,
@@ -50,13 +54,9 @@ class VariableRecord:
             # temporary requirement to support both new and old type objects
             # we divide by 32 here because the returned value is denominated
             # in "slots" of 32 bytes each
+            # CMC 20211023 revisit this divide-by-32.
             return math.ceil(self.typ.size_in_bytes / 32)
-        return get_size_of_type(self.typ)
-
-
-class ContractRecord(VariableRecord):
-    def __init__(self, *args):
-        super(ContractRecord, self).__init__(*args)
+        return math.ceil(self.typ.memory_bytes_required / 32)
 
 
 @dataclass
@@ -98,7 +98,7 @@ class FunctionSignature:
         return input_name + ":"
 
     @cached_property
-    def _lll_identifier(self) -> str:
+    def _ir_identifier(self) -> str:
         # we could do a bit better than this but it just needs to be unique
         visibility = "internal" if self.internal else "external"
         argz = ",".join([str(arg.typ) for arg in self.args])
@@ -108,7 +108,7 @@ class FunctionSignature:
     # calculate the abi signature for a given set of kwargs
     def abi_signature_for_kwargs(self, kwargs):
         args = self.base_args + kwargs
-        return self.name + "(" + ",".join([canonicalize_type(arg.typ) for arg in args]) + ")"
+        return self.name + "(" + ",".join([arg.typ.abi_type.selector_name() for arg in args]) + ")"
 
     @cached_property
     def base_signature(self):
@@ -119,17 +119,17 @@ class FunctionSignature:
     def external_function_base_entry_label(self):
         assert not self.internal
 
-        return self._lll_identifier + "_common"
+        return self._ir_identifier + "_common"
 
     @property
     def internal_function_label(self):
         assert self.internal, "why are you doing this"
 
-        return self._lll_identifier
+        return self._ir_identifier
 
     @property
     def exit_sequence_label(self):
-        return self._lll_identifier + "_cleanup"
+        return self._ir_identifier + "_cleanup"
 
     def set_default_args(self):
         """Split base from kwargs and set member data structures"""
@@ -166,7 +166,6 @@ class FunctionSignature:
             argname = arg.arg
             argtyp = parse_type(
                 arg.annotation,
-                None,
                 sigs,
                 custom_structs=custom_structs,
             )
@@ -205,12 +204,11 @@ class FunctionSignature:
         if func_ast.returns:
             return_type = parse_type(
                 func_ast.returns,
-                None,
                 sigs,
                 custom_structs=custom_structs,
             )
             # sanity check: Output type must be canonicalizable
-            assert canonicalize_type(return_type)
+            assert return_type.abi_type.selector_name()
 
         return cls(
             name,
