@@ -17,14 +17,15 @@ from vyper.exceptions import (
 from vyper.semantics.namespace import get_namespace
 from vyper.semantics.types.bases import BaseTypeDefinition, DataLocation, StorageSlot
 from vyper.semantics.types.indexable.sequence import TupleDefinition
-from vyper.semantics.types.user.struct import StructDefinition
 from vyper.semantics.types.utils import (
     StringEnum,
-    check_constant,
+    check_kwargable,
+    generate_abi_type,
     get_type_from_abi,
     get_type_from_annotation,
 )
-from vyper.semantics.types.value.numeric import Uint256Definition
+from vyper.semantics.types.value.boolean import BoolDefinition
+from vyper.semantics.types.value.numeric import Uint256Definition  # type: ignore
 from vyper.semantics.validation.utils import validate_expected_type
 from vyper.utils import keccak256
 
@@ -305,7 +306,7 @@ class ContractFunction(BaseTypeDefinition):
 
         namespace = get_namespace()
         for arg, value in zip(node.args.args, defaults):
-            if arg.arg in ("gas", "value"):
+            if arg.arg in ("gas", "value", "skip_contract_check"):
                 raise ArgumentException(
                     f"Cannot use '{arg.arg}' as a variable name in a function input",
                     arg,
@@ -322,17 +323,23 @@ class ContractFunction(BaseTypeDefinition):
                 arg.annotation, location=DataLocation.CALLDATA, is_constant=True
             )
             if value is not None:
-                if not check_constant(value):
+                if not check_kwargable(value):
                     raise StateAccessViolation(
                         "Value must be literal or environment variable", value
                     )
                 validate_expected_type(value, type_definition)
+                # kludge because kwargs in signatures don't get visited by the annotator
+                value._metadata["type"] = type_definition
 
             arguments[arg.arg] = type_definition
 
         # return types
         if node.returns is None:
             return_type = None
+        elif node.name == "__init__":
+            raise FunctionDeclarationException(
+                "Constructor may not have a return type", node.returns
+            )
         elif isinstance(node.returns, (vy_ast.Name, vy_ast.Call, vy_ast.Subscript)):
             return_type = get_type_from_annotation(node.returns, location=DataLocation.MEMORY)
         elif isinstance(node.returns, vy_ast.Tuple):
@@ -397,7 +404,7 @@ class ContractFunction(BaseTypeDefinition):
         * For functions with default arguments, there is one key for each
           function signature.
         """
-        arg_types = [i.canonical_type for i in self.arguments.values()]
+        arg_types = [i.canonical_abi_type for i in self.arguments.values()]
 
         if not self.has_default_args:
             return _generate_method_id(self.name, arg_types)
@@ -443,7 +450,7 @@ class ContractFunction(BaseTypeDefinition):
         # for external calls, include gas and value as optional kwargs
         kwarg_keys = self.kwarg_keys.copy()
         if node.get("func.value.id") != "self":
-            kwarg_keys += ["gas", "value"]
+            kwarg_keys += ["gas", "value", "skip_contract_check"]
         validate_call_args(node, (self.min_arg_count, self.max_arg_count), kwarg_keys)
 
         if self.mutability < StateMutability.PAYABLE:
@@ -457,6 +464,10 @@ class ContractFunction(BaseTypeDefinition):
         for kwarg in node.keywords:
             if kwarg.arg in ("gas", "value"):
                 validate_expected_type(kwarg.value, Uint256Definition())
+            elif kwarg.arg in ("skip_contract_check"):
+                validate_expected_type(kwarg.value, BoolDefinition())
+                if not isinstance(kwarg.value, vy_ast.NameConstant):
+                    raise InvalidType("skip_contract_check must be literal bool", kwarg.value)
             else:
                 validate_expected_type(kwarg.arg, kwarg.value)
 
@@ -475,15 +486,15 @@ class ContractFunction(BaseTypeDefinition):
             abi_dict["type"] = "function"
             abi_dict["name"] = self.name
 
-        abi_dict["inputs"] = [_generate_abi_type(v, k) for k, v in self.arguments.items()]
+        abi_dict["inputs"] = [generate_abi_type(v, k) for k, v in self.arguments.items()]
 
         typ = self.return_type
         if typ is None:
             abi_dict["outputs"] = []
         elif isinstance(typ, TupleDefinition) and len(typ.value_type) > 1:  # type: ignore
-            abi_dict["outputs"] = [_generate_abi_type(i) for i in typ.value_type]  # type: ignore
+            abi_dict["outputs"] = [generate_abi_type(i) for i in typ.value_type]  # type: ignore
         else:
-            abi_dict["outputs"] = [_generate_abi_type(typ)]
+            abi_dict["outputs"] = [generate_abi_type(typ)]
 
         if self.has_default_args:
             # for functions with default args, return a dict for each possible arg count
@@ -496,22 +507,53 @@ class ContractFunction(BaseTypeDefinition):
             return [abi_dict]
 
 
-def _generate_abi_type(type_definition, name=""):
-    if isinstance(type_definition, StructDefinition):
-        return {
-            "name": name,
-            "type": "tuple",
-            "components": [_generate_abi_type(v, k) for k, v in type_definition.members.items()],
-        }
-    if isinstance(type_definition, TupleDefinition):
-        return {
-            "type": "tuple",
-            "components": [_generate_abi_type(i) for i in type_definition.value_type],
-        }
-    return {"name": name, "type": type_definition.canonical_type}
+class MemberFunctionDefinition(BaseTypeDefinition):
+    """
+    Member function type definition.
+
+    This class has no corresponding primitive.
+
+    (examples for (x <DynArray[int128, 3]>).append(1))
+
+    Arguments:
+        underlying_type: the type this method is attached to. ex. DynArray[int128, 3]
+        name: the name of this method. ex. "append"
+        arg_types: the argument types this method accepts. ex. [int128]
+        return_type: the return type of this method. ex. None
+    """
+
+    _is_callable = True
+
+    def __init__(
+        self,
+        underlying_type: BaseTypeDefinition,
+        name: str,
+        arg_types: List[BaseTypeDefinition],
+        return_type: Optional[BaseTypeDefinition],
+        is_modifying: bool,
+    ) -> None:
+        super().__init__(DataLocation.UNSET)
+        self.underlying_type = underlying_type
+        self.name = name
+        self.arg_types = arg_types
+        self.return_type = return_type
+        self.is_modifying = is_modifying
+
+    def __repr__(self):
+        return f"{self.underlying_type._id} member function '{self.name}'"
+
+    def fetch_call_return(self, node: vy_ast.Call) -> Optional[BaseTypeDefinition]:
+        validate_call_args(node, len(self.arg_types))
+
+        assert len(node.args) == len(self.arg_types)  # validate_call_args postcondition
+        for arg, expected_type in zip(node.args, self.arg_types):
+            # CMC 2022-04-01 this should probably be in the validation module
+            validate_expected_type(arg, expected_type)
+
+        return self.return_type
 
 
-def _generate_method_id(name: str, canonical_types: List[str]) -> Dict[str, int]:
-    function_sig = f"{name}({','.join(canonical_types)})"
+def _generate_method_id(name: str, canonical_abi_types: List[str]) -> Dict[str, int]:
+    function_sig = f"{name}({','.join(canonical_abi_types)})"
     selector = keccak256(function_sig.encode())[:4].hex()
     return {function_sig: int(selector, 16)}
