@@ -23,8 +23,9 @@ from vyper.semantics.namespace import get_namespace
 from vyper.semantics.types.bases import DataLocation
 from vyper.semantics.types.function import ContractFunction
 from vyper.semantics.types.user.event import Event
-from vyper.semantics.types.utils import check_literal, get_type_from_annotation
+from vyper.semantics.types.utils import check_constant, get_type_from_annotation
 from vyper.semantics.validation.base import VyperNodeVisitorBase
+from vyper.semantics.validation.levenshtein_utils import get_levenshtein_error_suggestions
 from vyper.semantics.validation.utils import validate_expected_type, validate_unique_method_ids
 from vyper.typing import InterfaceDict
 
@@ -96,19 +97,22 @@ class ModuleNodeVisitor(VyperNodeVisitorBase):
         module_node._metadata["type"] = interface
 
         # get list of internal function calls made by each function
-        call_function_names = set()
-        for node in self.ast.get_children(vy_ast.FunctionDef):
-            call_function_names.add(node.name)
-            self_members[node.name].internal_calls = set(
+        function_defs = self.ast.get_children(vy_ast.FunctionDef)
+        function_names = set(node.name for node in function_defs)
+        for node in function_defs:
+            calls_to_self = set(
                 i.func.attr for i in node.get_descendants(vy_ast.Call, {"func.value.id": "self"})
             )
+            # anything that is not a function call will get semantically checked later
+            calls_to_self = calls_to_self.intersection(function_names)
+            self_members[node.name].internal_calls = calls_to_self
             if node.name in self_members[node.name].internal_calls:
                 self_node = node.get_descendants(
                     vy_ast.Attribute, {"value.id": "self", "attr": node.name}
                 )[0]
                 raise CallViolation(f"Function '{node.name}' calls into itself", self_node)
 
-        for fn_name in sorted(call_function_names):
+        for fn_name in sorted(function_names):
 
             if fn_name not in self_members:
                 # the referenced function does not exist - this is an issue, but we'll report
@@ -179,8 +183,17 @@ class ModuleNodeVisitor(VyperNodeVisitorBase):
                         vy_ast.Assign, filters={"target.id": node.target.id}
                     )
                     if not assignments:
+                        # Special error message for common wrong usages via `self.<immutable name>`
+                        wrong_self_attribute = self.ast.get_descendants(
+                            vy_ast.Attribute, {"value.id": "self", "attr": node.target.id}
+                        )
+                        message = (
+                            "Immutable variables must be accessed without 'self'"
+                            if len(wrong_self_attribute) > 0
+                            else "Immutable definition requires an assignment in the constructor"
+                        )
                         raise SyntaxException(
-                            "Immutable definition requires an assignment in the constructor",
+                            message,
                             node.node_source_code,
                             node.lineno,
                             node.col_offset,
@@ -198,7 +211,7 @@ class ModuleNodeVisitor(VyperNodeVisitorBase):
         if is_constant:
             if not node.value:
                 raise VariableDeclarationException("Constant must be declared with a value", node)
-            if not check_literal(node.value):
+            if not check_constant(node.value):
                 raise StateAccessViolation("Value must be a literal", node.value)
 
             validate_expected_type(node.value, type_definition)
@@ -293,7 +306,8 @@ def _add_import(
     if module == "vyper.interfaces":
         interface_codes = _get_builtin_interfaces()
     if name not in interface_codes:
-        raise UndeclaredDefinition(f"Unknown interface: {name}", node)
+        suggestions_str = get_levenshtein_error_suggestions(name, _get_builtin_interfaces(), 1.0)
+        raise UndeclaredDefinition(f"Unknown interface: {name}. {suggestions_str}", node)
 
     if interface_codes[name]["type"] == "vyper":
         interface_ast = vy_ast.parse_to_ast(interface_codes[name]["code"], contract_name=name)
